/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : buildgameslist.cpp
 */




#include "buildgameslist.h"
#include "options.h"
#include "mameprocess.h"

#include <qfile.h>
#include <qtextstream.h>
#include <qprocess.h>
#include <qapplication.h>


#define KAF_SPEED_TEST

#if defined( KAF_SPEED_TEST )
#include <qdatetime.h>
#endif

XmlGameParser::XmlGameParser() : QObject(), QXmlDefaultHandler()
{}

XmlGameParser::~XmlGameParser()
{}

bool XmlGameParser::startDocument()
{
  return true;
}

bool XmlGameParser::startElement( const QString& /*namespaceURI*/,
                                  const QString& /*localName*/,
                                  const QString& qName,
                                  const QXmlAttributes& atts )
{
  if( qName.isNull() )
    return true;

  m_state = XS_Unknown;

  QString sTmp = "";

  if( qName == "game" )
  {
    m_game.init();

    sTmp = atts.value( "name" );
    if( !sTmp.isNull() )
      m_game.name() = sTmp;

    sTmp = atts.value( "runnable" );
    if( sTmp.isNull() || sTmp != "no" )
      m_game.setFlag( Game::Runnable );

    sTmp = atts.value( "cloneof" );
    if( !sTmp.isNull() )
    {
      m_game.cloneof() = sTmp;
      m_game.setFlag( Game::Clone );
    }

    sTmp = atts.value( "romof" );
    if( !sTmp.isNull() )
      m_game.romof() = sTmp;
  }
  else if( qName == "description" )
  {
    m_state = XS_Description;
    m_game.description() = "";
  }
  else if( qName == "manufacturer" )
  {
    m_state = XS_Manufacturer;
    m_game.manufacturer() = "";
  }
  else if( qName == "year" )
  {
    m_state = XS_Year;
    m_game.year() = "";
  }
  else if( qName == "video" )
  {
    sTmp = atts.value( "screen" );
    if( !sTmp.isNull() )
    {
      if( sTmp.lower() == "vector" )
        m_game.setFlag( Game::Vector );
      else if( sTmp.lower() = "raster" )
        m_game.setFlag( Game::Raster );
      else
        qDebug( "Unknown video screen mode for game '%s'.", m_game.name().latin1() );
    }
  }
  else
  {
    /* Skip this piece of code */
  }

  return true;
}

bool XmlGameParser::characters( const QString& ch )
{
  if( ch.isNull() || ch.isEmpty() )
    return true;

  switch( m_state )
  {
  case XS_Description:
    m_game.description() += ch;
    break;

  case XS_Manufacturer:
    m_game.manufacturer() += ch;
    break;

  case XS_Year:
    m_game.year() += ch;
    break;

  default:
    /* Do nothing */
    return true;
  }

  return true;
}

bool XmlGameParser::endElement( const QString& /*namespaceURI*/,
                                const QString& /*localName*/,
                                const QString& qName )
{
  if( !qName.isNull() && qName == "game" )
  {
    emit( gameFound( m_game ) );
  }

  m_state = XS_Unknown;

  return true;
}

/**********************/

BuildGamesList::BuildGamesList( Options* pOptions, QObject* parent )
  : QObject( parent ), QThread()
{
  mp_options = pOptions;
  m_state = Init;
  m_error = "no error";
  mp_stream = 0;
}

BuildGamesList::~BuildGamesList()
{
  if( mp_stream )
    delete mp_stream;
}

void BuildGamesList::setState( int new_state )
{
  m_state = new_state;
  emit( stateChanged( m_state ) );
}

void BuildGamesList::setError( const QString& error )
{
  m_error = error;
  setState( Error );
  qWarning( m_error );
}

bool BuildGamesList::dumpToXmlFile()
{
#if defined( KAF_SPEED_TEST )
  qDebug( "Build Games List - dumping games list on xml file at %s.",
    QTime::currentTime().toString().latin1() );
#endif

  setState( Dumping );

  mp_options->setGroup( G_MAME );
  QString cmd = mp_options->getString( O_MAME_EXE, "xmame" );
  cmd += " -noloadconfig";
  cmd += " -lx > ";
  cmd += mp_options->mameXmlFilePath();
  QProcess *pMame = new QProcess( QString( "sh" ) );
  pMame->addArgument( "-c" );
  pMame->addArgument( cmd );

  if( !pMame->start() )
  {
    QString sTmp = "";
    sTmp.sprintf( "Cannot run binary '%s'.", cmd.latin1() );
    setError( sTmp );
    delete pMame;
    return false;
  }

  while( pMame->isRunning() )
  {
    QThread::msleep( 300 );
  }

#if defined( KAF_SPEED_TEST )
  qDebug( "Build Games List - games list dumped on xml file at %s.",
    QTime::currentTime().toString().latin1() );
#endif

  delete pMame;

  setState( Dumped );

  return true;
}

bool BuildGamesList::checkXmlFile( QFile* pFile )
{
  if( !pFile->exists() )
  {
    QString sTmp = "";
    sTmp.sprintf( "Unable to find file '%s'.", mp_options->mameXmlFilePath().latin1() );
    setError( sTmp );
    return false;
  }

  if( !pFile->open( IO_ReadOnly ) )
  {
    QString sTmp = "";
    sTmp.sprintf( "Unable to open file '%s'.", mp_options->mameXmlFilePath().latin1() );
    setError( sTmp );
    return false;
  }

  return true;
}


void BuildGamesList::saveGame( Game game )
{
  game.save( mp_stream );
}

void BuildGamesList::run()
{
#if defined( KAF_SPEED_TEST )
  qDebug( "Build Games List - start process at %s.",
    QTime::currentTime().toString().latin1() );
#endif

  MameProcess *mp = new MameProcess( mp_options );
  mp->checkVersion();
  mp->checkSupportedGames();
  delete mp;

  mp_options->setGroup( G_MAME );
  mp_options->setString( O_MAME_GAMES_LIST_VERSION, mp_options->getString( O_MAME_VERSION, "" ) );
  mp_options->setString( O_MAME_GAMES_LIST_KAF_VERSION, Options::getKafVersion() );

  dumpToXmlFile();

  setState( Parsing );

  QFile *pXmlFile = new QFile( mp_options->mameXmlFilePath() );

  if( !checkXmlFile( pXmlFile ) )
  {
    /* Error Occurred */
    delete pXmlFile;
    return;
  }

  QFile *pTxtFile = new QFile( mp_options->mameTxtFilePath() );
  if( !pTxtFile->open( IO_WriteOnly ) )
  {
    pXmlFile->close();
    pXmlFile->remove();
    delete pXmlFile;
    delete pTxtFile;
    return;
  }

  mp_stream = new QTextStream( pTxtFile );


  (*mp_stream) << mp_options->mameGamesListHeader() << endl << flush;

#if defined( KAF_SPEED_TEST )
  qDebug( "Build Games List - parsing xml file at %s.",
    QTime::currentTime().toString().latin1() );
#endif

  QXmlInputSource xml_source( pXmlFile );
  QXmlSimpleReader xml_reader;
  XmlGameParser xml_parser;
  connect( &xml_parser, SIGNAL( gameFound( Game ) ), this, SLOT( saveGame( Game ) ) );

  xml_reader.setContentHandler( &xml_parser );

  xml_reader.parse( xml_source );

#if defined( KAF_SPEED_TEST )
  qDebug( "Build Games List - xml file parsed at %s.",
    QTime::currentTime().toString().latin1() );
#endif

  setState( Parsed );

  pXmlFile->close();
  pXmlFile->remove();
  delete pXmlFile;

  pTxtFile->close();
  delete pTxtFile;
  delete mp_stream;
  mp_stream = 0;

#if defined( KAF_SPEED_TEST )
  qDebug( "Build Games List - process completed at %s.",
    QTime::currentTime().toString().latin1() );
#endif

  setState( Finished );
}


