/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : gameslist.cpp
 */




#include "gameslist.h"
#include "gameitem.h"
#include "game.h"

#include <qfile.h>
#include <qtextstream.h>

const QString KafAllGamesString = "* All games *";

GamesList::GamesList( QListView *pList )
{
  mp_list = pList;
  m_availables = 0;
  addCategory( KafAllGamesString );
  m_total = 0;
  m_categoryFilter = "";
}

GamesList::~GamesList()
{
}

void GamesList::clear()
{
  mp_list->clear();
  m_categories.clear();
  addCategory( KafAllGamesString );
  m_availables = 0;
  m_total = 0;
}

bool GamesList::saveFavorites( const QString& file_path )
{
  QFile file( file_path );
  if( !file.open( IO_WriteOnly ) )
    return false;

  qDebug( "Save favorite games in the file '%s'.", file_path.latin1() );
  QTextStream ts( &file );

  ts << m_favorites.join( "\n" ) << endl << flush;

  file.close();

  return true;
}

bool GamesList::loadFavorites( const QString& file_path )
{
  QFile file( file_path );
  if( !file.open( IO_ReadOnly ) )
    return false;

  if( !m_favorites.isEmpty() )
    m_favorites.clear();

  qDebug( "Load favorite games from the file '%s'.", file_path.latin1() );
  QTextStream ts( &file );
  QString sTmp = "";

  while( !ts.atEnd() )
  {
    sTmp = ts.readLine();
    if( !sTmp.isNull() && !sTmp.isEmpty() )
    {
      sTmp = sTmp.stripWhiteSpace();
      m_favorites.append( sTmp );
    }
  }

  file.close();

  return true;
}

void GamesList::addGame( Game* pGame )
{
  checkFlags( pGame );
  GameItem* pGameItem = new GameItem( mp_list, pGame );
  m_total++;
  pGameItem->setVisible( canBeShowedInList( pGameItem ) );
}

void GamesList::checkFlags( Game* pGame )
{
  if( pGame->hasFlag( Game::Available ) )
  {
    pGame->removeFlag( Game::Unavailable );
    m_availables++;
  }
  else
    pGame->setFlag( Game::Unavailable );

  if( isFavorite( pGame->name() ) )
    pGame->setFlag( Game::Favorite );
  else
    pGame->removeFlag( Game::Favorite );

  if( pGame->romof() == "neogeo" )
    pGame->setFlag( Game::NeoGeo );
  else
    pGame->removeFlag( Game::NeoGeo );

  if( !pGame->cloneof().isEmpty() )
  {
    pGame->setFlag( Game::Clone );
    pGame->removeFlag( Game::Original );
  }
  else
  {
    pGame->setFlag( Game::Original );
    pGame->removeFlag( Game::Clone );
  }

  addCategory( pGame->category() );
}

bool GamesList::canBeShowedInList( QListViewItem* item )
{
  GameItem* gi = (GameItem*)item;

  if( m_filter != 0 && !gi->game()->hasFlag( m_filter ) )
    return false;

  if( !m_categoryFilter.isEmpty() &&
      m_categoryFilter != KafAllGamesString &&
      gi->game()->category() != m_categoryFilter )
    return false;

  return true;
}

void GamesList::refreshFiltered()
{
  QListViewItemIterator it( mp_list );
  while( it.current() )
  {
    (it.current())->setVisible( canBeShowedInList( it.current() ) );
    ++it;
  }
}

void GamesList::refreshAllGames()
{
  QListViewItemIterator it( mp_list );
  GameItem *gi = NULL;
  while( it.current() )
  {
    gi = (GameItem*)it.current();
    gi->refreshFields();
    ++it;
  }
}

void GamesList::setAvailable( GameItem* item )
{
  Game *pGame = item->game();
  if( pGame->hasFlag( Game::Available ) )
  {
    //qDebug( "The game '%s' is already an available one.", pGame->name().latin1() );
    return;
  }

  pGame->setFlag( Game::Available );
  pGame->removeFlag( Game::Unavailable );
  item->refreshFields();
  m_availables++;
}

void GamesList::unsetAvailable( GameItem* item )
{
  Game *pGame = item->game();
  if( !pGame->hasFlag( Game::Available ) )
  {
    //qDebug( "The game '%s' is already an unavailable one.", pGame->name().latin1() );
    return;
  }

  pGame->removeFlag( Game::Available );
  pGame->setFlag( Game::Unavailable );
  item->refreshFields();
  m_availables--;
  if( m_availables < 0 )
  {
    qDebug( "The number of the available games was negative, now it is fixed." );
    m_availables = 0;
  }
}

bool GamesList::isFavorite( const QString& name )
{
  for( QStringList::Iterator it = m_favorites.begin(); it != m_favorites.end(); ++it )
  {
    if( *it == name )
      return true;
  }

  return false;
}

void GamesList::setFavorite( GameItem* item )
{
  Game *pGame = item->game();
  if( pGame->hasFlag( Game::Favorite ) )
  {
    //qDebug( "The game '%s' is already a favorite one.", pGame->name().latin1() );
    return;
  }

  pGame->setFlag( Game::Favorite );
  if( !isFavorite( pGame->name() ) )  // Prevents duplicates
    m_favorites.append( pGame->name() );
  item->refreshFields();
}

void GamesList::unsetFavorite( GameItem* item )
{
  Game *pGame = item->game();
  if( !pGame->hasFlag( Game::Favorite ) )
  {
    //qDebug( "The game '%s' is already an unfavorite one.", pGame->name().latin1() );
    return;
  }

  pGame->removeFlag( Game::Favorite );

  m_favorites.remove( pGame->name() );

  item->refreshFields();
}

Game* GamesList::game( const QString& game_name )
{
  QListViewItemIterator it( mp_list );
  while( it.current() )
  {
    GameItem* gi = (GameItem*)it.current();
    if( gi->game()->name() == game_name )
      return gi->game();
    ++it;
  }

  return NULL;
}

bool GamesList::hasCategory( const QString& cat_name )
{
  for( QStringList::Iterator it = m_categories.begin(); it != m_categories.end(); ++it )
  {
    if( *it == cat_name )
      return true;
  }

  return false;
}

void GamesList::addCategory( const QString& cat_name )
{
  if( !hasCategory( cat_name ) )
  {
    m_categories.append( cat_name );
    //qDebug( "Added game category '%s'.", cat_name.latin1() );
  }
}
