/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : options.cpp
 */



#include "options.h"
#include "version.h"

#include <qdir.h>
#include <qstringlist.h>


Options::Options() : KSettings( KAF_VERSION, "KAF" )
{
  QString optionsFilePath = personalDirectory();
  optionsFilePath += "/kaf.ini";
  setFile( optionsFilePath );

  if( !loadData() )
  {
    qWarning( "Unable to load preferences from the file '%s'.", optionsFilePath.latin1() );
    setDefaults();
  }
}

Options::~Options()
{
  if( m_dirty )
    flush();
}

QString Options::getKafVersion()
{
  return KAF_VERSION;
}

QString Options::getMameBasedVersion()
{
  return KAF_XMAME_BASED;
}

QString Options::mameVersion()
{
  QString prevGroup = group();
  setGroup( G_MAME );
  QString sRet = getString( O_MAME_VERSION, "" );
  setGroup( prevGroup );
  return sRet;
}

void Options::setDefaults()
{
  qDebug( "Restoring default options..." );

  setGroup( G_MAME );
  setString( O_MAME_EXE, "/usr/local/bin/xmame" );
  setString( O_MAME_ROMS_DIR, "/usr/local/games/mame/roms" );
  setString( O_MAME_ICONS_DIR, "/usr/local/games/mame/icons" );
  setString( O_MAME_SNAPS_DIR, "/usr/local/games/mame/snap" );
  setString( O_MAME_SAMPLES_DIR, "/usr/local/games/mame/samples" );
  setString( O_MAME_INI_DIR, "/usr/local/games/mame/ini" );
  setString( O_MAME_ARTWORK_DIR, "/usr/local/games/mame/artwork" );
  setString( O_MAME_FLYERS_DIR, "/usr/local/games/mame/flyers" );
  setString( O_MAME_CTRLR_DIR, "/usr/local/games/mame/ctrlr" );
  setString( O_MAME_CHEAT_FILE, "/usr/local/games/mame/cheat.dat" );
  setString( O_MAME_HISCORE_FILE, "/usr/local/games/mame/hiscore.dat" );
  setString( O_MAME_HISTORY_FILE, "/usr/local/games/mame/history.dat" );
  setString( O_MAME_INFO_FILE, "/usr/local/games/mame/mameinfo.dat" );

  setGroup( G_GENERAL );
  setString( O_CHECK_EVENT_TICK, "200" );
  setString( O_CHECK_PROGRESS_TICK, "40" );
  setString( O_PREVIEW_FILE_TYPE, "png" );

  flush();

  qDebug( "Default options restored." );
}

bool Options::checkEventTick( int tick )
{
  QString sPrevGroup = currentGroup();
  setGroup( G_MAME );
  int iCheck = getNumber( O_CHECK_EVENT_TICK, 200 );
  setGroup( sPrevGroup );
  return ( tick % iCheck == 0 );
}

bool Options::checkProgressTick( int tick )
{
  QString sPrevGroup = currentGroup();
  setGroup( G_MAME );
  int iCheck = getNumber( O_CHECK_PROGRESS_TICK, 40 );
  setGroup( sPrevGroup );
  return ( tick % iCheck == 0 );
}

QString Options::mameGamesListHeader()
{
  QString sPrevGroup = currentGroup();

  QString sHeader = "";

  setGroup( G_MAME );

  sHeader.sprintf( "#KAF#|%s|%s|%s",
    getString( O_MAME_GAMES_LIST_VERSION, "" ).latin1(),
    getString( O_MAME_GAMES_SUPPORTED, "" ).latin1(),
    getString( O_MAME_GAMES_LIST_KAF_VERSION, "1.0.0" ).latin1() );

  setGroup( sPrevGroup );

  return sHeader;
}

bool Options::setMameGamesListHeader( const QString& sHeader )
{
  qDebug( "Parsing GamesList header: '%s'...", sHeader.latin1() );
  if( !sHeader.startsWith( "#KAF#" ) )
  {
    qDebug( "Invalid header found." );
    return false;
  }

  QStringList args = QStringList::split( "|", sHeader, true );
  if( args.count() != 4 )
  {
    qWarning( "GamesList header format is unknown." );
    return false;
  }

  QString sPrevGroup = currentGroup();

  setGroup( G_MAME );
  setString( O_MAME_GAMES_LIST_VERSION, *(args.at( 1 )) );
  setString( O_MAME_GAMES_SUPPORTED, *(args.at( 2 )) );
  setString( O_MAME_GAMES_LIST_KAF_VERSION, *(args.at( 3 )) );

  qDebug( "GamesList header succefully parsed." );

  setGroup( sPrevGroup );
  return true;
}

QString Options::personalDirectory()
{
  QString sPrevGroup = currentGroup();
  setGroup( G_GENERAL );
  QString def = QDir::homeDirPath();
  def += "/.kaf";
  QString sDir = getString( O_PERSONAL_DIR, def );
  QDir dir( sDir );
  if( !dir.exists() )
  {
    if( !dir.mkdir( sDir, true ) )
      qWarning( "Cannot create directory '%s'.", sDir.latin1() );
  }
  setGroup( sPrevGroup );
  return sDir;
}

QString Options::tempDirectory()
{
  QString sPrevGroup = currentGroup();
  setGroup( G_GENERAL );
  QString def = personalDirectory();
  def += "/temp";
  QString sDir = getString( O_TEMP_DIR, def );
  QDir dir( sDir );
  if( !dir.exists() )
  {
    if( !dir.mkdir( sDir, true ) )
      qWarning( "Cannot create directory '%s'.", sDir.latin1() );
  }
  setGroup( sPrevGroup );
  return sDir;
}

QString Options::mameXmlFilePath()
{
  QString sPath = tempDirectory();
  sPath += "/mame.xml";
  return sPath;
}

QString Options::mameTxtFilePath()
{
  QString sPath = mameTempDirectory();
  sPath += "/gameslist.txt";
  return sPath;
}

QString Options::mameFavoritesFilePath()
{
  QString sPath = mameTempDirectory();
  sPath += "/favorites.txt";
  return sPath;
}

QString Options::mameTempDirectory()
{
  QString sPrevGroup = currentGroup();
  setGroup( G_GENERAL );
  QString sDir = personalDirectory();
  sDir += "/mame";
  QDir dir( sDir );
  if( !dir.exists() )
  {
    if( !dir.mkdir( sDir, true ) )
      qWarning( "Cannot create directory '%s'.", sDir.latin1() );
  }
  setGroup( sPrevGroup );
  return sDir;
}

QString Options::mamePreviewFileType()
{
   QString sPrevGroup = currentGroup();
   setGroup( G_GENERAL );
   QString sFileType = ".";
   sFileType += getString( O_PREVIEW_FILE_TYPE, "png" );
   setGroup( sPrevGroup );
   return sFileType;
}

QString Options::mameIconFileType()
{
   QString sPrevGroup = currentGroup();
   setGroup( G_GENERAL );
   QString sFileType = ".";
   sFileType += getString( O_ICON_FILE_TYPE, "ico" );
   setGroup( sPrevGroup );
   return sFileType;
}


