/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : wndgameoptions.cpp
 */




#include "wndgameoptions.h"
#include "options.h"
#include "pixmaps.h"

#include <qtabwidget.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qbuttongroup.h>
#include <qslider.h>
#include <qlabel.h>
#include <qvalidator.h>
#include <qapplication.h>
#include <qtextedit.h>
#include <qmessagebox.h>
#include <qradiobutton.h>


static int double2int( double d )
{
  QString sTmp = QString::number( d );
  return sTmp.toInt();
}



WndGameOptions::WndGameOptions( Options* pOptions, QWidget *parent )
  : GuiGameOptions( parent, "GameOptionsDialog", true )
{
  setIcon( Pixmaps::getMameIcon() );

  mp_options = pOptions;

  QDoubleValidator *pDoubleValidator = new QDoubleValidator( 0.50, 3.50, 2, this, "DoubleValidator" );
  QIntValidator *pIntValidator = new QIntValidator( this, "IntValidator" );

  leDisplayAspectRatio->setValidator( pDoubleValidator );
  lePauseBrightness->setValidator( pDoubleValidator );
  leArbScale->setValidator( pIntValidator );
  leServerPort->setValidator( pIntValidator );
  leUdpPort->setValidator( pIntValidator );

  connect( bOk, SIGNAL( clicked() ), this, SLOT( slotOk() ) );
  connect( bCancel, SIGNAL( clicked() ), this, SLOT( slotCancel() ) );
  connect( bRestore, SIGNAL( clicked() ), this, SLOT( slotRestoreDefaultOptions() ) );

  connect( cbX11Fullscreen, SIGNAL( clicked() ), this, SLOT( slotVideoModeSelected() ) );
  connect( cbKeepAspectRatio, SIGNAL( clicked() ), this, SLOT( slotAspectRatioSelected() ) );
  connect( bgX11Window, SIGNAL( clicked( int ) ), this, SLOT( slotX11WindowOptionSelected( int ) ) );

  connect( cbXGLUseCustomResolution, SIGNAL( clicked() ), this, SLOT( slotXGLCustomResolutionSelected() ) );
  connect( cbXGLTextureSize, SIGNAL( clicked() ), this, SLOT( slotXGLTextureSizeSelected() ) );

  connect( cbSdlCustomMode, SIGNAL( clicked() ), this, SLOT( slotSDLCustomModeSelected() ) );

  connect( sliderGamma, SIGNAL( valueChanged( int ) ), this, SLOT( slotGamma( int ) ) );
  connect( sliderBrightness, SIGNAL( valueChanged( int ) ), this, SLOT( slotBrightness( int ) ) );

  connect( cbDisableRotation, SIGNAL( clicked() ), this, SLOT( slotDisableRotationSelected() ) );
  connect( cbAutomaticFrameSkip, SIGNAL( clicked() ), this, SLOT( slotAutomaticFrameSkipSelected() ) );

  connect( cbEnableSound, SIGNAL( clicked() ), this, SLOT( slotEnableSoundSelected() ) );
  connect( sliderVolume, SIGNAL( valueChanged( int ) ), this, SLOT( slotVolume( int ) ) );
  connect( sliderSoundBuffer, SIGNAL( valueChanged( int ) ), this, SLOT( slotSoundBuffer( int ) ) );

  connect( cbEnableNetwork, SIGNAL( clicked() ), this, SLOT( slotEnableNetworkSelected() ) );
  connect( bgNetworkMode, SIGNAL( clicked( int ) ), this, SLOT( slotNetworkModeSelected( int ) ) );

  connect( cbVectorUseFixedResolution, SIGNAL( clicked() ), this, SLOT( slotUseFixedResolutionSelected() ) );
  connect( sliderBeam, SIGNAL( valueChanged( int ) ), this, SLOT( slotBeam( int ) ) );
  connect( sliderFlicker, SIGNAL( valueChanged( int ) ), this, SLOT( slotFlicker( int ) ) );
  connect( sliderIntensity, SIGNAL( valueChanged( int ) ), this, SLOT( slotIntensity( int ) ) );

  connect( cbUseArtwork, SIGNAL( clicked() ), this, SLOT( slotUseArtworkSelected() ) );

  updateMenus();
}

WndGameOptions::~WndGameOptions()
{}

void WndGameOptions::updateMenus()
{
  updateX11Menus();
  updateXGLMenus();
  updateSDLMenus();
  updateDisplayMenus();
  updateSoundMenus();
  updateInputMenus();
  updateNetworkMenus();
  updateVectorMenus();
  updateAdvancedMenus();

  updateRenderMenus();
}

void WndGameOptions::updateRenderMenus()
{
  mp_options->setGroup( G_MAME );
  QString sRender = mp_options->getString( O_MAME_RENDER, "x11" ).lower();
  if( sRender == "x11" )
  {
    twMain->removePage( tabXGL );
    twMain->removePage( tabRender );
  }
  else if( sRender == "xgl" )
  {
    twMain->removePage( tabX11 );
    twMain->removePage( tabRender );
  }
  else
  {
    twMain->removePage( tabX11 );
    twMain->removePage( tabXGL );
  }

  bgGlVector->setEnabled( ( sRender == "xgl" ) );
  bgSdl->setEnabled( ( sRender == "sdl" ) );
  cbGrabMouse->setEnabled( ( sRender != "sdl" ) );
  cbGrabKeyboard->setEnabled( ( sRender != "sdl" ) );
  cbUseWinkeys->setEnabled( ( sRender != "sdl" ) );
}

void WndGameOptions::updateX11Menus()
{
  mp_options->setGroup( G_MAME_X11 );
  comboX11VideoMode->setCurrentItem( mp_options->getNumber( O_X11_VIDEO_MODE, 1 ) );
  cbX11Fullscreen->setChecked( mp_options->getBool( O_X11_FULLSCREEN, false ) );
  slotVideoModeSelected();
  sbX11Width->setValue( mp_options->getNumber( O_X11_GEOMETRY_WIDTH, 640 ) );
  sbX11Height->setValue( mp_options->getNumber( O_X11_GEOMETRY_HEIGHT, 480 ) );
  cbKeepAspectRatio->setChecked( mp_options->getBool( O_X11_KEEP_ASPECT_RATIO, true ) );
  slotAspectRatioSelected();
  leDisplayAspectRatio->setText( QString::number( mp_options->getDouble( O_X11_DISPLAY_ASPECT_RATIO, 1.33 ) ) );
  leDisableMode->setText( mp_options->getString( O_X11_DISABLE_MODE, "" ) );
  cbShowCursor->setChecked( mp_options->getBool( O_X11_SHOW_CURSOR, true ) );
  cbUseMit->setChecked( mp_options->getBool( O_X11_MIT, true ) );
  cbForceYuv->setChecked( mp_options->getBool( O_X11_FORCE_YUV, false ) );
  cbForceYv12->setChecked( mp_options->getBool( O_X11_FORCE_YV12, false ) );
  cbUseWidescreen->setChecked( mp_options->getBool( O_X11_WIDESCREEN, false ) );
  cbUseXSync->setChecked( mp_options->getBool( O_X11_XSYNC, false ) );
  cbUsePrivateColorMap->setChecked( mp_options->getBool( O_X11_PRIVATE_COLOR_MAP, false ) );
  cbUseXil->setChecked( mp_options->getBool( O_X11_XIL, true ) );
  cbUseXilMultiThreading->setChecked( mp_options->getBool( O_X11_XIL_MULTITHREAD, true ) );
}

void WndGameOptions::updateXGLMenus()
{
  mp_options->setGroup( G_MAME_XGL );
  cbXGLFullscreen->setChecked( mp_options->getBool( O_XGL_FULLSCREEN, false ) );
  cbXGLUseCustomResolution->setChecked( mp_options->getBool( O_XGL_CUSTOM_RESOLUTION, true ) );
  sbXGLWidth->setValue( mp_options->getNumber( O_XGL_GEOMETRY_WIDTH, 640 ) );
  sbXGLHeight->setValue( mp_options->getNumber( O_XGL_GEOMETRY_HEIGHT, 480 ) );
  slotXGLCustomResolutionSelected();
  cbXGLTextureSize->setChecked( mp_options->getBool( O_XGL_USE_TEXTURE_SIZE, false ) );
  sbXGLTextureSize->setValue( mp_options->getNumber( O_XGL_TEXTURE_SIZE, 0 ) );
  slotXGLTextureSizeSelected();
  cbXGLUseDoubleBuffering->setChecked( mp_options->getBool( O_XGL_DOUBLE_BUFFERING, true ) );
  cbXGLUseAlphablending->setChecked( mp_options->getBool( O_XGL_ALPHA_BLENDING, true ) );
  cbXGLEnableAntialias->setChecked( mp_options->getBool( O_XGL_ANTIALIAS, true ) );
  cbXGLUseBilinearFilter->setChecked( mp_options->getBool( O_XGL_BILINEAR_FILTER, true ) );
  cbXGLUseColorModulation->setChecked( mp_options->getBool( O_XGL_COLOR_MODULATION, true ) );
  cbXGLUseBlitter->setChecked( mp_options->getBool( O_XGL_FORCE_BLITTER, true ) );
  cbXGLDrawBitmap->setChecked( mp_options->getBool( O_XGL_DRAW_BITMAP, true ) );
  cbXGLExt78->setChecked( mp_options->getBool( O_XGL_FORCE_EXT78, true ) );
  leOpenGLPath->setText( mp_options->getString( O_XGL_OPENGL_PATH, "" ) );
  leGLUPath->setText( mp_options->getString( O_XGL_GLU_PATH, "" ) );
}

void WndGameOptions::updateSdlMode()
{
  /*  Fill combo box of the SDL mode number option.
   *  We can obtain the list of options launching xmame -listmodes.
   */
  if( comboSdlMode->count() > 0 )
    comboSdlMode->clear();

  comboSdlMode->insertItem( "1024 x 768", 0 );
  comboSdlMode->insertItem( "960 x 720", 1 );
  comboSdlMode->insertItem( "928 x 696", 2 );
  comboSdlMode->insertItem( "896 x 672", 3 );
  comboSdlMode->insertItem( "800 x 600", 4 );
  comboSdlMode->insertItem( "700 x 525", 5 );
  comboSdlMode->insertItem( "640 x 512", 6 );
  comboSdlMode->insertItem( "800 x 512", 7 );
  comboSdlMode->insertItem( "640 x 480", 8 );
  comboSdlMode->insertItem( "640 x 400", 9 );
  comboSdlMode->insertItem( "720 x 400", 10 );
  comboSdlMode->insertItem( "640 x 350", 11 );
  comboSdlMode->insertItem( "576 x 432", 12 );
  comboSdlMode->insertItem( "512 x 384", 13 );
  comboSdlMode->insertItem( "576 x 384", 14 );
  comboSdlMode->insertItem( "416 x 312", 15 );
  comboSdlMode->insertItem( "320 x 240", 16 );
}

void WndGameOptions::updateSDLMenus()
{
  updateSdlMode();

  mp_options->setGroup( G_MAME_SDL );

  cbSdlFullscreen->setChecked( mp_options->getBool( O_SDL_FULLSCREEN, false ) );
  cbSdlCustomMode->setChecked( mp_options->getBool( O_SDL_CUSTOM_MODE, false ) );
  comboSdlMode->setCurrentItem( mp_options->getNumber( O_SDL_MODE, 15 ) );

  slotSDLCustomModeSelected();
}

void WndGameOptions::updateDisplayMenus()
{
  mp_options->setGroup( G_DISPLAY );

  comboVideoEffects->setCurrentItem( mp_options->getNumber( O_DISPLAY_EFFECTS, 0 ) );
  //cbUseScanlines->setChecked( mp_options->getBool( O_DISPLAY_SCANLINES, false ) );
  cbSleepIdle->setChecked( mp_options->getBool( O_DISPLAY_SLEEP_IDLE, false ) );

  double dTmp = mp_options->getDouble( O_DISPLAY_GAMMA, 1.0 ) * 100;
  int iTmp = double2int( dTmp );
  sliderGamma->setValue( iTmp );
  slotGamma( iTmp );
  dTmp = mp_options->getDouble( O_DISPLAY_BRIGHTNESS, 1.0 ) * 100;
  iTmp = double2int( dTmp );
  sliderBrightness->setValue( iTmp );
  slotBrightness( iTmp );
  lePauseBrightness->setText( QString::number( mp_options->getDouble( O_DISPLAY_PAUSE_BRIGHTNESS, 1.65 ) ) );

  cbDisableRotation->setChecked( mp_options->getBool( O_DISPLAY_DISABLE_ROTATION, false ) );
  comboRotation->setCurrentItem( mp_options->getNumber( O_DISPLAY_ROTATION, 0 ) );
  cbFlipLR->setChecked( mp_options->getBool( O_DISPLAY_FLIP_LR, false ) );
  cbFlipUD->setChecked( mp_options->getBool( O_DISPLAY_FLIP_UD, false ) );
  slotDisableRotationSelected();

  sbXScale->setValue( mp_options->getNumber( O_DISPLAY_X_SCALE, 1 ) );
  sbYScale->setValue( mp_options->getNumber( O_DISPLAY_Y_SCALE, 1 ) );
  comboBpp->setCurrentItem( mp_options->getNumber( O_DISPLAY_BPP, 0 ) );
  cbAutoDouble->setChecked( mp_options->getBool( O_DISPLAY_AUTO_DOUBLE, true ) );
  leArbScale->setText( QString::number( mp_options->getNumber( O_DISPLAY_ARG_SCALE, 0 ) ) );

  cbAutomaticFrameSkip->setChecked( mp_options->getBool( O_DISPLAY_AUTO_FRAME_SKIP, true ) );
  sbFrameSkip->setValue( mp_options->getNumber( O_DISPLAY_NUM_FRAMES_TO_SKIP, 8 ) );
  sbMaxFrameSkip->setValue( mp_options->getNumber( O_DISPLAY_MAX_NUM_FRAMES_TO_SKIP, 8 ) );
  comboFrameSkip->setCurrentItem( mp_options->getNumber( O_DISPLAY_FRAME_SKIP_METHOD, 0 ) );
  cbTrottle->setChecked( mp_options->getBool( O_DISPLAY_THROTTLE, true ) );
  slotAutomaticFrameSkipSelected();
}

void WndGameOptions::updateSoundMenus()
{
  mp_options->setGroup( G_SOUND );

  cbEnableSound->setChecked( mp_options->getBool( O_SOUND_ENABLE, true ) );
  slotEnableSoundSelected();
  cbUseFakeSound->setChecked( mp_options->getBool( O_SOUND_FAKE, false ) );
  cbUseSamples->setChecked( mp_options->getBool( O_SOUND_SAMPLES, true ) );
  comboSamplerate->setCurrentText( mp_options->getString( O_SOUND_SAMPLERATE, "44100" ) );
  int iTmp = -(mp_options->getNumber( O_SOUND_VOLUME, -3 ));
  sliderVolume->setValue( iTmp );
  slotVolume( iTmp );
  double dTmp = mp_options->getDouble( O_SOUND_BUFFER, 3.0 ) * 10;
  iTmp = double2int( dTmp );
  sliderSoundBuffer->setValue( iTmp );
  slotSoundBuffer( iTmp );
  leAudioDevice->setText( mp_options->getString( O_SOUND_AUDIO_DEVICE, "" ) );
  leMixerDevice->setText( mp_options->getString( O_SOUND_MIXER_DEVICE, "" ) );
  leDspPlugin->setText( mp_options->getString( O_SOUND_DSP_PLUGIN, "" ) );
  leMixerPlugin->setText( mp_options->getString( O_SOUND_MIXER_PLUGIN, "" ) );
}

void WndGameOptions::updateInputMenus()
{
  mp_options->setGroup( G_INPUT );

  cbGrabMouse->setChecked( mp_options->getBool( O_INPUT_GRAB_MOUSE, false ) );
  cbGrabKeyboard->setChecked( mp_options->getBool( O_INPUT_GRAB_KEYBOARD, false ) );
  cbUseWinkeys->setChecked( mp_options->getBool( O_INPUT_WINKEYS, false ) );
  leKeymap->setText( mp_options->getString( O_INPUT_KEYMAP, "" ) );
  comboJoystickType->setCurrentItem( mp_options->getNumber( O_INPUT_JOYSTICK_TYPE, 0 ) );
  cbUseAnalogJoystick->setChecked( mp_options->getBool( O_INPUT_ANALOG_JOYSTICK, false ) );
  leJoystickDevice->setText( mp_options->getString( O_INPUT_JOYSTICK_DEVICE, "/dev/js" ) );
  cbUseMouse->setChecked( mp_options->getBool( O_INPUT_MOUSE, true ) );
  cbUseUgci->setChecked( mp_options->getBool( O_INPUT_UGCI, false ) );
  cbUseUsbPad->setChecked( mp_options->getBool( O_INPUT_USB_PAD, false ) );
  cbUseRapidFire->setChecked( mp_options->getBool( O_INPUT_RAPID_FIRE, false ) );
  leControllerPreconfigure->setText( mp_options->getString( O_INPUT_CONTROLLER_CFG, "" ) );
}

void WndGameOptions::updateNetworkMenus()
{
  mp_options->setGroup( G_NETWORK );

  cbEnableNetwork->setChecked( mp_options->getBool( O_NETWORK_ENABLE, false ) );
  slotEnableNetworkSelected();

  int iTmp = ( mp_options->getString( O_NETWORK_MODE, "server" ) == "client" ) ? 1 : 0;
  bgNetworkMode->setButton( iTmp );
  slotNetworkModeSelected( iTmp );

  sbPlayers->setValue( mp_options->getNumber( O_NETWORK_PLAYERS, 2 ) );
  cbParallelSync->setChecked( mp_options->getBool( O_NETWORK_SYNC, true ) );
  cbNetworkDebug->setChecked( mp_options->getBool( O_NETWORK_DEBUG, false ) );

  leServerName->setText( mp_options->getString( O_NETWORK_SERVERNAME, "" ) );
  leServerPort->setText( mp_options->getString( O_NETWORK_SERVERPORT, "" ) );
  cbNetmapkey->setChecked( mp_options->getBool( O_NETWORK_NETMAPKEY, true ) );

  leUdpPort->setText( mp_options->getString( O_NETWORK_UDP_PORT, "9000" ) );
}

void WndGameOptions::updateVectorMenus()
{
  mp_options->setGroup( G_VECTOR );

  cbVectorAntialias->setChecked( mp_options->getBool( O_VECTOR_ANTIALIAS, true ) );
  cbVectorTranslucency->setChecked( mp_options->getBool( O_VECTOR_TRANSLUCENCY, true ) );

  double dTmp = mp_options->getDouble( O_VECTOR_BEAM, 1.0 ) * 100;
  int iTmp = double2int( dTmp );
  sliderBeam->setValue( iTmp );
  slotBeam( iTmp );

  dTmp = mp_options->getDouble( O_VECTOR_FLICKER, 0.0 );
  iTmp = double2int( dTmp );
  sliderFlicker->setValue( iTmp );
  slotFlicker( iTmp );

  dTmp = mp_options->getDouble( O_VECTOR_INTENSITY, 1.5 ) * 100;
  iTmp = double2int( dTmp );
  sliderIntensity->setValue( iTmp );
  slotIntensity( iTmp );

  cbVectorUseFixedResolution->setChecked( mp_options->getBool( O_VECTOR_FIXED_RESOLUTION, false ) );
  comboVectorResolution->setCurrentText( mp_options->getString( O_VECTOR_RESOLUTION, "320x200" ) );
  slotUseFixedResolutionSelected();

  cbGlVectorDrawBitmap->setChecked( mp_options->getBool( O_VECTOR_GL_DRAW_BITMAP, true ) );
  cbGlVectorAntialias->setChecked( mp_options->getBool( O_VECTOR_GL_ANTIALIAS, true ) );
}

void WndGameOptions::updateAdvancedMenus()
{
  mp_options->setGroup( G_MAME );
  teCustomOptions->setText( mp_options->getString( O_MAME_CUSTOM_OPTIONS, "" ) );
  cbUseOnlyCustomOptions->setChecked( mp_options->getBool( O_MAME_ONLY_CUSTOM_OPTIONS, false ) );
  teRemoveOptions->setText( mp_options->getString( O_MAME_REMOVE_OPTIONS, "" ) );

  mp_options->setGroup( G_MISC );
  cbUseCheat->setChecked( mp_options->getBool( O_MISC_CHEAT, false ) );
  cbSkipDisclaimer->setChecked( mp_options->getBool( O_MISC_SKIP_DISCLAIMER, false ) );
  cbSkipGameInfo->setChecked( mp_options->getBool( O_MISC_SKIP_GAME_INFO, false ) );
  cbUseArtwork->setChecked( mp_options->getBool( O_MISC_ARTWORK, false ) );
  cbUseBackdrops->setChecked( mp_options->getBool( O_MISC_BACKDROPS, false ) );
  cbUseOverlays->setChecked( mp_options->getBool( O_MISC_OVERLAYS, false ) );
  cbUseBezels->setChecked( mp_options->getBool( O_MISC_BEZELS, false ) );
  cbCropArtwork->setChecked( mp_options->getBool( O_MISC_CROP_ARTWORK, false ) );
  //#define O_MISC_ARTWORK_RESOLUTION    "Artwork resolution"
  slotUseArtworkSelected();
}

void WndGameOptions::slotOk()
{
  double dTmp = 1.0;

  mp_options->setGroup( G_MAME );
  mp_options->setString( O_MAME_CUSTOM_OPTIONS, teCustomOptions->text().simplifyWhiteSpace() );
  mp_options->setBool( O_MAME_ONLY_CUSTOM_OPTIONS, cbUseOnlyCustomOptions->isChecked() );
  mp_options->setString( O_MAME_REMOVE_OPTIONS, teRemoveOptions->text().simplifyWhiteSpace() );

  mp_options->setGroup( G_MAME_X11 );
  mp_options->setNumber( O_X11_VIDEO_MODE, comboX11VideoMode->currentItem() );
  mp_options->setBool( O_X11_FULLSCREEN,cbX11Fullscreen->isChecked() );
  mp_options->setNumber( O_X11_GEOMETRY_WIDTH, sbX11Width->value() );
  mp_options->setNumber( O_X11_GEOMETRY_HEIGHT, sbX11Height->value() );

#if QT_VERSION >= 0x030100
  QRect rect = qApp->desktop()->availableGeometry();
#else
  QRect rect = qApp->desktop()->screenGeometry();
#endif
  int ioW = ( rect.width() - sbX11Width->value() ) / 2;
  int ioH = ( rect.height() - sbX11Height->value() ) / 2;
  mp_options->setNumber( O_X11_OFFSET_WIDTH, ioW );
  mp_options->setNumber( O_X11_OFFSET_HEIGHT, ioH );

  mp_options->setBool( O_X11_KEEP_ASPECT_RATIO, cbKeepAspectRatio->isChecked() );
  mp_options->setDouble( O_X11_DISPLAY_ASPECT_RATIO, leDisplayAspectRatio->text().toDouble() );
  mp_options->setString( O_X11_DISABLE_MODE, leDisableMode->text() );
  mp_options->setBool( O_X11_SHOW_CURSOR, cbShowCursor->isChecked() );
  mp_options->setBool( O_X11_MIT, cbUseMit->isChecked() );
  mp_options->setBool( O_X11_FORCE_YUV, cbForceYuv->isChecked() );
  mp_options->setBool( O_X11_FORCE_YV12, cbForceYv12->isChecked() );
  mp_options->setBool( O_X11_WIDESCREEN, cbUseWidescreen->isChecked() );
  mp_options->setBool( O_X11_XSYNC, cbUseXSync->isChecked() );
  mp_options->setBool( O_X11_PRIVATE_COLOR_MAP, cbUsePrivateColorMap->isChecked() );
  mp_options->setBool( O_X11_XIL, cbUseXil->isChecked() );
  mp_options->setBool( O_X11_XIL_MULTITHREAD, cbUseXilMultiThreading->isChecked() );

  mp_options->setGroup( G_MAME_XGL );
  mp_options->setBool( O_XGL_FULLSCREEN, cbXGLFullscreen->isChecked() );
  mp_options->setBool( O_XGL_CUSTOM_RESOLUTION, cbXGLUseCustomResolution->isChecked() );
  mp_options->setNumber( O_XGL_GEOMETRY_WIDTH, sbXGLWidth->value() );
  mp_options->setNumber( O_XGL_GEOMETRY_HEIGHT, sbXGLHeight->value() );
  ioW = ( rect.width() - sbXGLWidth->value() ) / 2;
  ioH = ( rect.height() - sbXGLHeight->value() ) / 2;
  mp_options->setNumber( O_XGL_OFFSET_WIDTH, ioW );
  mp_options->setNumber( O_XGL_OFFSET_HEIGHT, ioH );
  mp_options->setBool( O_XGL_USE_TEXTURE_SIZE, cbXGLTextureSize->isChecked() );
  mp_options->setNumber( O_XGL_TEXTURE_SIZE, sbXGLTextureSize->value() );
  mp_options->setBool( O_XGL_DOUBLE_BUFFERING, cbXGLUseDoubleBuffering->isChecked() );
  mp_options->setBool( O_XGL_ALPHA_BLENDING, cbXGLUseAlphablending->isChecked() );
  mp_options->setBool( O_XGL_ANTIALIAS, cbXGLEnableAntialias->isChecked() );
  mp_options->setBool( O_XGL_BILINEAR_FILTER, cbXGLUseBilinearFilter->isChecked() );
  mp_options->setBool( O_XGL_COLOR_MODULATION, cbXGLUseColorModulation->isChecked() );
  mp_options->setBool( O_XGL_FORCE_BLITTER, cbXGLUseBlitter->isChecked() );
  mp_options->setBool( O_XGL_DRAW_BITMAP, cbXGLDrawBitmap->isChecked() );
  mp_options->setBool( O_XGL_FORCE_EXT78, cbXGLExt78->isChecked() );
  mp_options->setString( O_XGL_OPENGL_PATH, leOpenGLPath->text().simplifyWhiteSpace() );
  mp_options->setString( O_XGL_GLU_PATH, leGLUPath->text().simplifyWhiteSpace() );

  mp_options->setGroup( G_MAME_SDL );
  mp_options->setBool( O_SDL_FULLSCREEN, cbSdlFullscreen->isChecked() );
  mp_options->setBool( O_SDL_CUSTOM_MODE, cbSdlCustomMode->isChecked() );
  mp_options->setNumber( O_SDL_MODE, comboSdlMode->currentItem() );


  mp_options->setGroup( G_DISPLAY );
  mp_options->setNumber( O_DISPLAY_EFFECTS, comboVideoEffects->currentItem() );
  //mp_options->setBool( O_DISPLAY_SCANLINES, cbUseScanlines->isChecked() );
  mp_options->setBool( O_DISPLAY_SLEEP_IDLE, cbSleepIdle->isChecked() );
  dTmp = sliderGamma->value() / 100.0;
  mp_options->setDouble( O_DISPLAY_GAMMA, dTmp );
  dTmp = sliderBrightness->value() / 100.0;
  mp_options->setDouble( O_DISPLAY_BRIGHTNESS, dTmp );
  mp_options->setDouble( O_DISPLAY_PAUSE_BRIGHTNESS,lePauseBrightness->text().toDouble() );
  mp_options->setBool( O_DISPLAY_DISABLE_ROTATION, cbDisableRotation->isChecked() );
  mp_options->setNumber( O_DISPLAY_ROTATION, comboRotation->currentItem() );
  mp_options->setBool( O_DISPLAY_FLIP_LR, cbFlipLR->isChecked() );
  mp_options->setBool( O_DISPLAY_FLIP_UD, cbFlipUD->isChecked() );
  mp_options->setNumber( O_DISPLAY_X_SCALE, sbXScale->value() );
  mp_options->setNumber( O_DISPLAY_Y_SCALE, sbYScale->value() );
  mp_options->setNumber( O_DISPLAY_BPP, comboBpp->currentItem() );
  mp_options->setBool( O_DISPLAY_AUTO_DOUBLE, cbAutoDouble->isChecked() );
  mp_options->setNumber( O_DISPLAY_ARG_SCALE, leArbScale->text().toInt() );
  mp_options->setBool( O_DISPLAY_AUTO_FRAME_SKIP, cbAutomaticFrameSkip->isChecked() );
  mp_options->setNumber( O_DISPLAY_NUM_FRAMES_TO_SKIP, sbFrameSkip->value() );
  mp_options->setNumber( O_DISPLAY_MAX_NUM_FRAMES_TO_SKIP, sbMaxFrameSkip->value() );
  mp_options->setNumber( O_DISPLAY_FRAME_SKIP_METHOD, comboFrameSkip->currentItem() );
  mp_options->setBool( O_DISPLAY_THROTTLE, cbTrottle->isChecked() );

  mp_options->setGroup( G_SOUND );
  mp_options->setBool( O_SOUND_ENABLE, cbEnableSound->isChecked() );
  mp_options->setBool( O_SOUND_FAKE, cbUseFakeSound->isChecked() );
  mp_options->setBool( O_SOUND_SAMPLES, cbUseSamples->isChecked() );
  mp_options->setString( O_SOUND_SAMPLERATE, comboSamplerate->currentText() );
  mp_options->setNumber( O_SOUND_VOLUME, -(sliderVolume->value( )) );
  dTmp = sliderSoundBuffer->value() / 10.0;
  mp_options->setDouble( O_SOUND_BUFFER, dTmp );
  mp_options->setString( O_SOUND_AUDIO_DEVICE, leAudioDevice->text() );
  mp_options->setString( O_SOUND_MIXER_DEVICE, leMixerDevice->text() );
  mp_options->setString( O_SOUND_DSP_PLUGIN, leDspPlugin->text() );
  mp_options->setString( O_SOUND_MIXER_PLUGIN, leMixerPlugin->text() );

  mp_options->setGroup( G_INPUT );
  mp_options->setBool( O_INPUT_GRAB_MOUSE, cbGrabMouse->isChecked() );
  mp_options->setBool( O_INPUT_GRAB_KEYBOARD, cbGrabKeyboard->isChecked() );
  mp_options->setBool( O_INPUT_WINKEYS, cbUseWinkeys->isChecked() );
  mp_options->setString( O_INPUT_KEYMAP, leKeymap->text().simplifyWhiteSpace() );
  mp_options->setNumber( O_INPUT_JOYSTICK_TYPE, comboJoystickType->currentItem() );
  mp_options->setBool( O_INPUT_ANALOG_JOYSTICK, cbUseAnalogJoystick->isChecked() );
  mp_options->setString( O_INPUT_JOYSTICK_DEVICE, leJoystickDevice->text().simplifyWhiteSpace() );
  mp_options->setBool( O_INPUT_MOUSE, cbUseMouse->isChecked() );
  mp_options->setBool( O_INPUT_UGCI, cbUseUgci->isChecked() );
  mp_options->setBool( O_INPUT_USB_PAD, cbUseUsbPad->isChecked() );
  mp_options->setBool( O_INPUT_RAPID_FIRE, cbUseRapidFire->isChecked() );
  mp_options->setString( O_INPUT_CONTROLLER_CFG, leControllerPreconfigure->text().simplifyWhiteSpace() );

  mp_options->setGroup( G_NETWORK );
  mp_options->setBool( O_NETWORK_ENABLE, cbEnableNetwork->isChecked() );
  if( bgNetworkMode->id( bgNetworkMode->selected() ) == 1 )
    mp_options->setString( O_NETWORK_MODE, "client" );
  else
    mp_options->setString( O_NETWORK_MODE, "server" );
  mp_options->setNumber( O_NETWORK_PLAYERS, sbPlayers->value() );
  mp_options->setBool( O_NETWORK_SYNC, cbParallelSync->isChecked() );
  mp_options->setBool( O_NETWORK_DEBUG, cbNetworkDebug->isChecked() );
  mp_options->setString( O_NETWORK_SERVERNAME, leServerName->text().simplifyWhiteSpace() );
  mp_options->setString( O_NETWORK_SERVERPORT, leServerPort->text().simplifyWhiteSpace() );
  mp_options->setBool( O_NETWORK_NETMAPKEY, cbNetmapkey->isChecked() );
  mp_options->setString( O_NETWORK_UDP_PORT, leUdpPort->text().simplifyWhiteSpace() );

  mp_options->setGroup( G_VECTOR );
  mp_options->setBool( O_VECTOR_ANTIALIAS, cbVectorAntialias->isChecked() );
  mp_options->setBool( O_VECTOR_TRANSLUCENCY, cbVectorTranslucency->isChecked() );
  dTmp = sliderBeam->value() / 100.0;
  mp_options->setDouble( O_VECTOR_BEAM, dTmp );
  dTmp = sliderFlicker->value();
  mp_options->setDouble( O_VECTOR_FLICKER, dTmp );
  dTmp = sliderIntensity->value() / 100.0;
  mp_options->setDouble( O_VECTOR_INTENSITY, dTmp );
  mp_options->setBool( O_VECTOR_FIXED_RESOLUTION, cbVectorUseFixedResolution->isChecked() );
  mp_options->setString( O_VECTOR_RESOLUTION, comboVectorResolution->currentText().simplifyWhiteSpace() );
  mp_options->setBool( O_VECTOR_GL_DRAW_BITMAP, cbGlVectorDrawBitmap->isChecked() );
  mp_options->setBool( O_VECTOR_GL_ANTIALIAS, cbGlVectorAntialias->isChecked() );

  mp_options->setGroup( G_MISC );
  mp_options->setBool( O_MISC_CHEAT, cbUseCheat->isChecked() );
  mp_options->setBool( O_MISC_SKIP_DISCLAIMER, cbSkipDisclaimer->isChecked() );
  mp_options->setBool( O_MISC_SKIP_GAME_INFO, cbSkipGameInfo->isChecked() );
  mp_options->setBool( O_MISC_ARTWORK, cbUseArtwork->isChecked() );
  mp_options->setBool( O_MISC_BACKDROPS, cbUseBackdrops->isChecked() );
  mp_options->setBool( O_MISC_OVERLAYS, cbUseOverlays->isChecked() );
  mp_options->setBool( O_MISC_BEZELS, cbUseBezels->isChecked() );
  mp_options->setBool( O_MISC_CROP_ARTWORK, cbCropArtwork->isChecked() );
  //#define O_MISC_ARTWORK_RESOLUTION    "Artwork resolution"

  mp_options->flush();

  done( 1 );
}

void WndGameOptions::slotCancel()
{
  done( 0 );
}

void WndGameOptions::slotVideoModeSelected()
{
  //frameX11Geometry->setEnabled( cbX11Fullscreen->isChecked() );
}

void WndGameOptions::slotAspectRatioSelected()
{
  leDisplayAspectRatio->setEnabled( cbKeepAspectRatio->isChecked() );
}

void WndGameOptions::slotX11WindowOptionSelected( int index )
{
  if( index == 2 )
  {
    if( cbForceYuv->isChecked() )
      cbForceYv12->setChecked( false );
  }

  if( index == 3 )
  {
    if( cbForceYv12->isChecked() )
      cbForceYuv->setChecked( false );
  }

  if( index == 7 )
    cbUseXilMultiThreading->setEnabled( cbUseXil->isChecked() );
}

void WndGameOptions::slotGamma( int gamma )
{
  float g = gamma / 100.0;
  labelGamma->setText( QString::number( g ) );
}

void WndGameOptions::slotBrightness( int brightness )
{
  float b = brightness / 100.0;
  labelBrightness->setText( QString::number( b ) );
}

void WndGameOptions::slotDisableRotationSelected()
{
  bool disable = !cbDisableRotation->isChecked();
  comboRotation->setEnabled( disable );
  cbFlipLR->setEnabled( disable );
  cbFlipUD->setEnabled( disable );
}

void WndGameOptions::slotAutomaticFrameSkipSelected()
{
  bool enable = cbAutomaticFrameSkip->isChecked();

  label8->setEnabled( !enable );
  sbFrameSkip->setEnabled( !enable );

  comboFrameSkip->setEnabled( enable );
  label7->setEnabled( enable );
  sbMaxFrameSkip->setEnabled( enable );
}

void WndGameOptions::slotEnableSoundSelected()
{
  frameSound->setEnabled( cbEnableSound->isChecked() );
}

void WndGameOptions::slotVolume( int volume )
{
  labelVolume->setText( QString::number( (-volume) ) + " db" );
}

void WndGameOptions::slotSoundBuffer( int buffer )
{
  labelSoundBuffer->setText( QString::number( (float)(buffer/10.0) ) + " frames" );
}

void WndGameOptions::slotEnableNetworkSelected()
{
  bgNetworkMode->setEnabled( cbEnableNetwork->isChecked() );
}

void WndGameOptions::slotNetworkModeSelected( int id )
{
  if( id == 1 ) // client mode
  {
    frameServerMode->setEnabled( false );
    frameClientMode->setEnabled( true );
  }
  else
  {
    frameServerMode->setEnabled( true );
    frameClientMode->setEnabled( false );
  }
}

void WndGameOptions::slotXGLCustomResolutionSelected()
{
  frameXGLResolution->setEnabled( cbXGLUseCustomResolution->isChecked() );
}

void WndGameOptions::slotXGLTextureSizeSelected()
{
  sbXGLTextureSize->setEnabled( cbXGLTextureSize->isChecked() );
}

void WndGameOptions::slotUseArtworkSelected()
{
  bgArtwork->setEnabled( cbUseArtwork->isChecked() );
}

void WndGameOptions::slotUseFixedResolutionSelected()
{
  comboVectorResolution->setEnabled( cbVectorUseFixedResolution->isChecked() );
}

void WndGameOptions::slotBeam( int value )
{
  double d = value / 100.0;
  labelBeam->setText( QString::number( d ) );
}

void WndGameOptions::slotFlicker( int value )
{
  labelFlicker->setText( QString::number( value ) );
}

void WndGameOptions::slotIntensity( int value )
{
  double d = value / 100.0;
  labelIntensity->setText( QString::number( d ) );
}

void WndGameOptions::slotRestoreDefaultOptions()
{
  switch( QMessageBox::information( this, trUtf8( "KAF - Information" ),
            trUtf8( "Do you really want to restore all default options?" ),
            trUtf8( "Yes" ), trUtf8( "No" ), QString::null, 1, 1 ) )
  {
  case 0:
    /* Go ahead */
    break;
  default:
    return;
  }

  mp_options->setGroup( G_MAME_X11 );
  mp_options->removeGroup();
  mp_options->setGroup( G_MAME_XGL );
  mp_options->removeGroup();
  mp_options->setGroup( G_MAME_SDL );
  mp_options->removeGroup();
  mp_options->setGroup( G_DISPLAY );
  mp_options->removeGroup();
  mp_options->setGroup( G_SOUND );
  mp_options->removeGroup();
  mp_options->setGroup( G_NETWORK );
  mp_options->removeGroup();
  mp_options->setGroup( G_INPUT );
  mp_options->removeGroup();
  mp_options->setGroup( G_VECTOR );
  mp_options->removeGroup();
  mp_options->setGroup( G_MISC );
  mp_options->removeGroup();

  updateMenus();
}

void WndGameOptions::slotSDLCustomModeSelected()
{
  comboSdlMode->setEnabled( cbSdlCustomMode->isChecked() );
}
